;**************************************************************************
;* FileName	: MAX7456.INC
;* Date 	: 14.08.2009
;* File version : v1.00
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: functions for MAX7456 OSD IC
;*		  ============================
;*		  MAX7456_init
;*		  MAX7456_clear
;*		  MAX7456_gotoxy
;*		  MAX7456_printc
;*		  MAX7456_printcs 
;*		  MAX7456_defchar
;*
;*		  MAX7456_mode_inverted
;*		  MAX7456_mode_not_inverted
;*		  MAX7456_mode_blinking
;*		  MAX7456_mode_not_blinking
;*		  MAX7456_mode_background
;*		  MAX7456_mode_transparent
;*
;*		  MAX7456_write_reg
;*		  MAX7456_read_reg
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* xtal_freq	EQU	4000000	  	; crystal frequency
;* core_freq	EQU	xtal_freq/4	; core operating frequency
;*
;* #define		MAX7456_SDIN	PORTB,0		; out
;* #define		MAX7456_SDOUT	PORTB,1		; in
;* #define		MAX7456_SCLK	PORTB,2		; out
;* #define		MAX7456_CS	PORTB,3		; out
;*
;*	include "..\modules\wait.mac"
;*	include "..\modules\math.mac"
;*	include "..\modules\wait.inc"
;*	include "..\modules\math.inc"
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* <none>
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  14/08/2009 	Initial Release
;*
;**************************************************************************

;***** VARIABLES **********************************************************

	cblock
	  MAX7456_addr				; address
	  MAX7456_data				; data
	  MAX7456_loopcount			; used for loops
	  MAX7456_posx				; x position
	  MAX7456_posy				; y position
	  MAX7456_dispaddr16:2			; display memory address
	  MAX7456_modeflag			; modeflag
	  MAX7456_progmem_addrhigh
	  MAX7456_progmem_addrlow
	  MAX7456_defchar_data
	  MAX7456_defchar_loopcounter
	endc




;***** MAX7456_init *******************************************************
;* initialize the MAX7456
;* Input : -					;
;* Output: - 					;
;* Clock : 50mS
;**************************************************************************
MAX7456_init 	bank1
		bcf	MAX7456_SDIN		; out
		bsf	MAX7456_SDOUT		; in
		bcf	MAX7456_SCLK		; out
		bcf	MAX7456_CS		; out		
		bank0
		bsf	MAX7456_SDIN		; high
		bcf	MAX7456_SCLK		; low
		bsf	MAX7456_CS		; high

		clrf	MAX7456_posx		; set position to (0,0)
		clrf	MAX7456_posy
		clrf	MAX7456_dispaddr16
		clrf	MAX7456_dispaddr16+1
		clrf	MAX7456_modeflag

MAX7456_init_wait
		MAX7456_READ_REG   MAX7456_REG_STATUS			; waiting for the end of reset (typ. 50mS)
		andlw	01000000b
		jnz	MAX7456_init_wait
		
		MAX7456_WRITE_REG  MAX7456_REG_VIDEO_MODE0, 01001000b	; 0, 1=PAL, 00=Auto Sync, 1=Enable OSD, 0=Enable OSD immer, 0=SW RST, 0=Enable Video Buffer 	
		MAX7456_WRITE_REG  MAX7456_REG_VIDEO_MODE1, 00010100b	; 0 = NA, 001=Background MODE Brightness 7%, 11=Blinking Time 160ms, 00=Blinking Duty Cycle BT:BT 
		MAX7456_WRITE_REG  MAX7456_REG_DISP_MODE,   00000000b	; 0, 0=16bit, 0=transparen, 0=not blinking, 0=not inverted, 00=no clear, 0=no auto increment

		MAX7456_READ_REG   MAX7456_REG_OSD_BLACK_LEVEL		; enable automatic OSD black level control
		andlw	11101111b 			
		MAX7456_WRITE_REGW MAX7456_REG_OSD_BLACK_LEVEL

		return




;***** MAX7456_clear ******************************************************
;* clear display
;* Input : -					;
;* Output: - 					;
;* Clock : ?		
;**************************************************************************
MAX7456_clear	
		MAX7456_WRITE_REG MAX7456_REG_DISP_MODE, 00000100b

		clrf	MAX7456_posx		; set position to (0,0)
		clrf	MAX7456_posy

		WAIT	20 Microsec, 0
		return 




;***** MAX7456_gotoxy *****************************************************
;* change cursor position
;* Input : MAX7456_posx				; x position (0...MAX7456_XSIZE-1)
;*	   MAX7456_posy				; y position (0...MAX7456_YSIZE-1)
;* Output: - 					;
;* Clock : ?		
;**************************************************************************
MAX7456_gotoxy	clrf	MAX7456_dispaddr16
		movf	MAX7456_posy,w
		andlw	00001111b		; limit to 0..15
		movwf	MAX7456_dispaddr16+1
		swapf	MAX7456_dispaddr16+1,f	; MAX7456_dispaddr16 = 16 * MAX7456_posy
		subwf	MAX7456_dispaddr16+1,f	; MAX7456_dispaddr16 = 15 * MAX7456_posy
		clrc	
		rlf	MAX7456_dispaddr16+1,f 
		rlf	MAX7456_dispaddr16,f 	; MAX7456_dispaddr16 = 30 * MAX7456_posy
		
		movf	MAX7456_posx,w
		addwf	MAX7456_dispaddr16+1,f
		skipnc
		incf	MAX7456_dispaddr16,f	; MAX7456_dispaddr16 = 30 * MAX7456_posy + MAX7456_posx 
	
		movf	MAX7456_dispaddr16,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_ADDRH

		movf	MAX7456_dispaddr16+1,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_ADDRL

		return




;***** MAX7456_printc *****************************************************
;* print an ASCII character to the LCD
;* Input : W <- char      			; ASCII code of the char
;* Output: - 					;
;* Clock : ?		
;**************************************************************************
MAX7456_printc	call	MAX7456_char_encode	; encode ASCII to MAX7456
		MAX7456_WRITE_REGW MAX7456_REG_DISP_DATAIN

		incf	MAX7456_dispaddr16+1,f	; INC16 MAX7456_dispaddr16
		jnz	MAX7456_pc_update_lowaddr_only

		incf	MAX7456_dispaddr16,f
		movf	MAX7456_dispaddr16,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_ADDRH

MAX7456_pc_update_lowaddr_only
		movf	MAX7456_dispaddr16+1,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_ADDRL

;-------------------------------------------------------------------------
		incf	MAX7456_posx,f		; increment posx

		movlw	MAX7456_XSIZE		; if (MAX7456_posx != MAX7456_XSIZE) return
		xorwf	MAX7456_posx,w	
		skipz
		return
		
		clrf	MAX7456_posx		; set position to the begin of next line
		clrc
		incf	MAX7456_posy,f

		movlw	MAX7456_YSIZE		; if (MAX7456_posy != MAX7456_YSIZE) return
		xorwf	MAX7456_posy,w	
		skipz
		return

		clrf	MAX7456_posy		; set position back to (0,0)
		clrf	MAX7456_dispaddr16
		clrf	MAX7456_dispaddr16+1

		movf	MAX7456_dispaddr16,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_ADDRH

		movf	MAX7456_dispaddr16+1,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_ADDRL

		return




;***** MAX7456_printcs ****************************************************
;* Print a constant string to the OSD
;* Input : w <- addr_high			; high address of cstring
;* Output: -					;
;* Clock : ?		
;**************************************************************************
MAX7456_printcs	movwf	MAX7456_progmem_addrhigh

MAX7456_printloop
		call	MAX7456_print_getc
		testw
		skipnz
		return

		call	MAX7456_printc

		incf	MAX7456_progmem_addrlow,f
		skipnz
		incf	MAX7456_progmem_addrhigh,f
		goto	MAX7456_printloop

;--------------------------------------------------------------------------
MAX7456_print_getc	
		movf	MAX7456_progmem_addrhigh,w
		movwf	PCLATH
		movf	MAX7456_progmem_addrlow,w
		movwf	PCL			; computed goto!





;***** MAX7456_mode_xxx ***************************************************
;* set inverted, blinking and transparent mode
;* Input : -					;
;* Output: - 					;
;* Clock : ?		
;**************************************************************************
MAX7456_mode_inverted
		movlw	MAX7456_CHARMODE_INVERTED
		iorwf	MAX7456_modeflag,f
		goto	MAX7456_setmode

MAX7456_mode_not_inverted
		movlw	MAX7456_CHARMODE_INVERTED
		xorlw	0FFh		
		andwf	MAX7456_modeflag,f
		goto	MAX7456_setmode
		
MAX7456_mode_blinking
		movlw	MAX7456_CHARMODE_BLINKING
		iorwf	MAX7456_modeflag,f
		goto	MAX7456_setmode

MAX7456_mode_not_blinking
		movlw	MAX7456_CHARMODE_BLINKING
		xorlw	0FFh		
		andwf	MAX7456_modeflag,f
		goto	MAX7456_setmode

MAX7456_mode_background
		movlw	MAX7456_CHARMODE_BACKGROUND
		iorwf	MAX7456_modeflag,f
		goto	MAX7456_setmode

MAX7456_mode_transparent
		movlw	MAX7456_CHARMODE_BACKGROUND
		xorlw	0FFh		
		andwf	MAX7456_modeflag,f
		goto	MAX7456_setmode



;***** MAX7456_setmode ****************************************************
;* set character mode
;* Input : MAX7456_modeflag			; character mode
;* Output: - 					;
;* Clock : ?		
;**************************************************************************
MAX7456_setmode	movf	MAX7456_modeflag,w
		MAX7456_WRITE_REGW MAX7456_REG_DISP_MODE
		return




;***** MAX7456_defchar ****************************************************
;* define a custom character
;* Input : w <- charcode		; MAX7456 character code (77..235)
;*         MAX7456_progmem_addrhigh	; high address of character pattern
;*         MAX7456_progmem_addrlow	; low address of character pattern
;* Output: - 				; 
;* Clock : about 12mS
;**************************************************************************
MAX7456_defchar movwf	MAX7456_defchar_data

		CMPC8	MAX7456_defchar_data, 77	; check valid range
		JB	MAX7456_defchar_end
		CMPC8	MAX7456_defchar_data, 235
		JA	MAX7456_defchar_end

		MAX7456_WRITE_REG  MAX7456_REG_VIDEO_MODE0, 01000000b	; Disable OSD

		movf	MAX7456_defchar_data,w				
		MAX7456_WRITE_REGW MAX7456_REG_CMEM_ADDRH		; select character 

		clrf	MAX7456_defchar_loopcounter

;--------------------------------------------------------------------------
MAX7456_defchar_loop
		movf	MAX7456_defchar_loopcounter,w				
		MAX7456_WRITE_REGW MAX7456_REG_CMEM_ADDRL		; select 4 pixel to be written 

		call	MAX7456_getchardata				
		MAX7456_WRITE_REGW MAX7456_REG_CMEM_DATAIN		; write pixel data

		incf	MAX7456_progmem_addrlow,f			; increment progmem address
		skipnz
		incf	MAX7456_progmem_addrhigh,f

		incf	MAX7456_defchar_loopcounter,f			; incremenet loop counter
		movf	MAX7456_defchar_loopcounter,w
		xorlw	54
		jnz	MAX7456_defchar_loop

;--------------------------------------------------------------------------
		MAX7456_WRITE_REG  MAX7456_REG_CMEM_MODE, 10100000b	; write to the NVM array from the shadow RAM
		
MAX7456_defchar_wait
		MAX7456_READ_REG   MAX7456_REG_STATUS			; waiting for the end of write (typ. 12mS)
		andlw	00100000b
		jnz	MAX7456_defchar_wait

		MAX7456_WRITE_REG  MAX7456_REG_VIDEO_MODE0, 01001000b	; Enable OSD

MAX7456_defchar_end
		return

;--------------------------------------------------------------------------
MAX7456_getchardata 
		movf	MAX7456_progmem_addrhigh,w
		movwf	PCLATH
		movf	MAX7456_progmem_addrlow,w
		movwf	PCL		; computed goto!




;***** MAX7456_write_reg **************************************************
;* Write register of MAX7456
;* Input : MAX7456_data				; data register
;*	 : W 					; register address
;* Output: - 					;
;* Clock : ?		
;**************************************************************************
MAX7456_write_reg 
		movwf	MAX7456_addr

		movlw	16
		movwf	MAX7456_loopcount

		bsf	MAX7456_CS		; CS=1
		bcf	MAX7456_SCLK		; SCLK=0

		bcf	MAX7456_CS		; CS=0

;-------------------------------------------------------------------------
MAX7456_wrloop	bcf	MAX7456_SDIN		; SDIN=0
		btfsc	MAX7456_addr,7		; test MSB	
		bsf	MAX7456_SDIN		; SDIN=1
		
		bsf	MAX7456_SCLK		; SCLK=1
		nop
		bcf	MAX7456_SCLK		; SCLK=0

		clrc				; SHL16 MAX7456_addr:MAX7456_data
		rlf	MAX7456_data,f
		rlf	MAX7456_addr,f 

		decfsz	MAX7456_loopcount,f
		goto	MAX7456_wrloop

		bsf	MAX7456_CS		; CS=1
		bcf	MAX7456_SDIN		; SDIN=0
		return




;***** MAX7456_read_reg **************************************************
;* read register of MAX7456
;* Input : W 				; register address
;* Output: W 				; data
;* Clock : ?		
;**************************************************************************
MAX7456_read_reg
		addlw	80h
		movwf	MAX7456_data

		movlw	8
		movwf	MAX7456_loopcount

		bsf	MAX7456_CS		; CS=1
		bcf	MAX7456_SCLK		; SCLK=0

		bcf	MAX7456_CS		; CS=0

;-------------------------------------------------------------------------
MAX7456_rrloop1	bcf	MAX7456_SDIN		; SDIN=0
		btfsc	MAX7456_data,7		; test MSB	
		bsf	MAX7456_SDIN		; SDIN=1

		bsf	MAX7456_SCLK		; SCLK=1
		nop
		bcf	MAX7456_SCLK		; SCLK=0

		clrc
		rlf	MAX7456_data,f 

		decfsz	MAX7456_loopcount,f
		goto	MAX7456_rrloop1	

;-------------------------------------------------------------------------
		bcf	MAX7456_SDIN		; SDIN=0

		movlw	8
		movwf	MAX7456_loopcount
		clrf	MAX7456_data

;-------------------------------------------------------------------------
MAX7456_rrloop2	clrc
		rlf	MAX7456_data,f 

		bsf	MAX7456_SCLK		; SCK=1
		btfsc	MAX7456_SDOUT		; test input pin
		bsf	MAX7456_data,0		; set bit
		bcf	MAX7456_SCLK		; SCK=0

		decfsz	MAX7456_loopcount,f
		goto	MAX7456_rrloop2

;-------------------------------------------------------------------------

		bsf	MAX7456_CS		; CS=1
		movf	MAX7456_data,w
		return




;***** MAX7456_char_encode ************************************************
;* encode character code
;*   ASCII 000..127 -> encode to MAX7456 character code
;*   ASCII 128..255 -> keep unchanged
;*
;* Input : W	           		; ASCII code of the char
;* Output: W				; MAX7456 character code
;* Clock : ?
;**************************************************************************
MAX7456_char_encode
		movwf	MAX7456_data

		movlw	HIGH MAX7456_char_table
		movwf	PCLATH

		movf	MAX7456_data,w
		addlw	LOW MAX7456_char_table	; set CARRY
		skipnc
		incf	PCLATH,f
		movwf	PCL

MAX7456_char_table
		retlw 0xFF					; ASCII 0x00
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0x10
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0x00		; space			; ASCII 0x20
		retlw 0x5F		; !
		retlw 0x48		; "
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x46		; '
		retlw 0x3F		; (
		retlw 0x40		; )
		retlw 0xFF
		retlw 0xFF
		retlw 0x45		; ,
		retlw 0x49		; -
		retlw 0x41		; .
		retlw 0x47		; /

		retlw 0x0A		; 0			; ASCII 0x30
		retlw 0x01		; 1
		retlw 0x02		; 2
		retlw 0x03		; 3
		retlw 0x04		; 4
		retlw 0x05		; 5
		retlw 0x06		; 6
		retlw 0x07		; 7
		retlw 0x08		; 8
		retlw 0x09		; 9
		retlw 0x44		; :
		retlw 0x43		; ;
		retlw 0x4A		; <
		retlw 0xFF
		retlw 0x4B		; >
		retlw 0x42		; 

		retlw 0x4C		; @			; ASCII 0x40
		retlw 0x0B		; A
		retlw 0x0C		; B
		retlw 0x0D		; C
		retlw 0x0E		; D
		retlw 0x0F		; E
		retlw 0x10		; F
		retlw 0x11		; G
		retlw 0x12		; H
		retlw 0x13		; I
		retlw 0x14		; J
		retlw 0x15		; K
		retlw 0x16		; L
		retlw 0x17		; M
		retlw 0x18		; N
		retlw 0x19		; O

		retlw 0x1A		; P			; ASCII 0x50
		retlw 0x1B		; Q
		retlw 0x1C		; R
		retlw 0x1D		; S
		retlw 0x1E		; T
		retlw 0x1F		; U
		retlw 0x20		; V
		retlw 0x21		; W
		retlw 0x22		; X
		retlw 0x23		; Y
		retlw 0x24		; Z
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0x60
		retlw 0x25		; a
		retlw 0x26		; b
		retlw 0x27		; c
		retlw 0x28		; d
		retlw 0x29		; e
		retlw 0x2A		; f
		retlw 0x2B		; g
		retlw 0x2C		; h
		retlw 0x2D		; i
		retlw 0x2E		; j
		retlw 0x2F		; k
		retlw 0x30		; l
		retlw 0x31		; m
		retlw 0x32		; n
		retlw 0x33		; o

		retlw 0x34		; p			; ASCII 0x70
		retlw 0x35		; q
		retlw 0x36		; r
		retlw 0x37		; s
		retlw 0x38		; t
		retlw 0x39		; u
		retlw 0x3A		; v
		retlw 0x3B		; w
		retlw 0x3C		; x
		retlw 0x3D		; y
		retlw 0x3E		; z
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xEC		; backward		; ASCII 0x80
		retlw 0xED		; bar begin
		retlw 0xEE		; bar full
		retlw 0xEF		; bar 3/4
		retlw 0xF0		; bar 2/4
		retlw 0xF1		; bar 1/4
		retlw 0xF2		; bar empty
		retlw 0xF3		; bar end
		retlw 0xF4		; forward
		retlw 0xF5		; brightness 1
		retlw 0xF6		; brightness 2
		retlw 0xF7		; contrast 1
		retlw 0xF8		; contrast 2
		retlw 0xF9		; volume 1
		retlw 0xFA		; volume 2
		retlw 0xFB		; hour-glass

		retlw 0xFC		; pointer		; ASCII 0x90
		retlw 0xFD		; AM
		retlw 0xFE		; PM
		retlw 0xFF		; full char
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0xA0
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0xB0
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0xC0
		retlw 0x4D    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x4E    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x4F    ; 
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0xD0
		retlw 0xFF
		retlw 0xFF
		retlw 0x50    ; 
		retlw 0xFF
		retlw 0x51    ; 
		retlw 0x52    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x53    ; 
		retlw 0x54    ; 
		retlw 0x55    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0xE0
		retlw 0x56    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x57    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x58    ; 
		retlw 0xFF
		retlw 0xFF

		retlw 0xFF					; ASCII 0xF0
		retlw 0xFF
		retlw 0xFF
		retlw 0x59    ; 
		retlw 0xFF
		retlw 0x5A    ; 
		retlw 0x5B    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF
		retlw 0x5C    ; 
		retlw 0x5D    ; 
		retlw 0x5E    ; 
		retlw 0xFF
		retlw 0xFF
		retlw 0xFF					; ASCII 0xFF



;**************************************************************************

